package restapi.utils.security.auth;

import com.google.common.collect.ImmutableMap;
import restapi.crud.UserRepository;
import restapi.services.UserService;
import restapi.services.UserSessionService;
import restapi.utils.security.token.TokenService;

import lombok.AllArgsConstructor;
import lombok.NonNull;
import lombok.experimental.FieldDefaults;
import org.springframework.stereotype.Service;

import java.util.Optional;

import static lombok.AccessLevel.PRIVATE;

@Service
@AllArgsConstructor
@FieldDefaults(level = PRIVATE, makeFinal = true)
final class TokenAuthenticationService implements UserAuthenticationService {
    @NonNull
    TokenService tokens;
    @NonNull
    UserRepository userRepository;
    @NonNull
    UserService userService;
    @NonNull
    UserSessionService userSessionService;

    @Override
    public Optional<String> login(final String username, final String password) {
        return userRepository
                .findByUsername(username)
                .filter(user -> userService.matchesEncrypted(password, user.getPassword()))
                .map(user -> {
                    String token = tokens.expiring(ImmutableMap.of("username", username));
                    userSessionService.reportLogin(user, token);
                    return token;
                });
    }


    @Override
    public Optional<String> login(String username) {
        return userRepository
                .findByUsername(username)
                .map(user -> {
                    String token = tokens.expiring(ImmutableMap.of("username", username));
                    userSessionService.reportLogin(user, token);
                    return token;
                });
    }

    @Override
    public Optional<LoggedUser> findByToken(final String token) {
        return Optional
                .of(tokens.verify(token))
                .map(map -> map.get("username"))
                .flatMap(userRepository::findByUsername)
                .map(userEntity -> userSessionService.getValidSession(userEntity, token))
                ;
                /*{
                    if (userEntity.getToken() == null) {
                        return null;
                    } else {
                        if (!userSessionService.isTokenInDB(userEntity, token) || userSessionService.isSessionExpired(userEntity)) {
                            return null;
                        } else {
                            return LoggedUser.builder()
                                    .username(userEntity.getUsername())
                                    .password(userEntity.getPassword()).build();
                        }
                    }
                });*/
    }

    @Override
    public void logout(final LoggedUser user) {
        userSessionService.logout(user.getUsername());
    }
}