package restapi.utils.services.date;

import org.joda.time.DateTime;
import org.joda.time.DateTimeZone;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

import java.util.TimeZone;

import static com.google.common.base.Preconditions.checkNotNull;

final class JodaDateService implements DateService {

    private final DateTimeZone timeZone;

    /**
     * Force system-wide timezone to ensure consistent
     * dates over all servers, independently from the region
     * the server is running.
     */
    JodaDateService(final DateTimeZone timeZone) {
        super();
        this.timeZone = checkNotNull(timeZone);

        System.setProperty("user.timezone", timeZone.getID());
        TimeZone.setDefault(timeZone.toTimeZone());
        DateTimeZone.setDefault(timeZone);
    }

    @Override
    public DateTime now() {
        return DateTime.now(timeZone);
    }

    @Override
    public String nowToString() {
        return dateToString(now());
    }

    @Override
    public String dateToString(DateTime dateTime) {
        DateTimeFormatter fmt = DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss");
        return dateTime.toString(fmt);
    }

    @Override
    public DateTime dateFromString(String date) {
        DateTimeFormatter fmt = DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss");
        return fmt.parseDateTime(date);
    }

    @Override
    public boolean isOlderThan(DateTime dateTime, int min){
        DateTime now = now();
        return dateTime.plusMinutes(min).isBefore(now);
    }

    @Override
    public boolean isOlderThan(String dateTime, int min) {
        return isOlderThan(dateFromString(dateTime), min);
    }
}