package restapi;

import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import restapi.json.ContactDTO;
import restapi.json.MatchRequestDTO;
import restapi.json.Token;
import restapi.services.FirebaseService;

import java.io.StringWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static org.hamcrest.Matchers.containsString;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc
@TestMethodOrder(MethodOrderer.Alphanumeric.class)
public class MatchEndpointTest {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private ObjectMapper objectMapper;

    @MockBean
    private FirebaseService firebaseService;

    private static final String username1 = "userMatchRequest1";
    private static final String username2 = "userMatchRequest2";
    private static final String username3 = "userMatchRequest3";
    private static final String password = "pas";
    private static String token1;
    private static String token2;
    private static String token3;
    private static long matchId12;
    private static long matchId23;
    private static long matchId13;

    @AllArgsConstructor
    @NoArgsConstructor
    public static class Matches {
        @Getter
        @Setter
        ArrayList<MatchRequestDTO> active;
        @Getter
        @Setter
        ArrayList<MatchRequestDTO> received;
        @Getter
        @Setter
        ArrayList<MatchRequestDTO> sent;
    }

    @Test
    public void AaSetUp() throws Exception {
        // Create users
        String body = mockMvc.perform(post("/public/users/signup")
                .param("username", username1)
                .param("password", password)
        ).andDo(print()).andExpect(status().isCreated()).andReturn().getResponse().getContentAsString();
        token1 = objectMapper.readValue(body, Token.class).getToken();
        body = mockMvc.perform(post("/public/users/signup")
                .param("username", username2)
                .param("password", password)
        ).andDo(print()).andExpect(status().isCreated()).andReturn().getResponse().getContentAsString();
        token2 = objectMapper.readValue(body, Token.class).getToken();
        body = mockMvc.perform(post("/public/users/signup")
                .param("username", username3)
                .param("password", password)
        ).andDo(print()).andExpect(status().isCreated()).andReturn().getResponse().getContentAsString();
        token3 = objectMapper.readValue(body, Token.class).getToken();
        // Make them friends
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token1);
        mockMvc.perform(post("/requests/new").headers(headers).param("username", username2))
                .andDo(print()).andExpect(status().isOk()).andExpect(content().string(containsString("Request")));
        mockMvc.perform(post("/requests/new").headers(headers).param("username", username3))
                .andDo(print()).andExpect(status().isOk()).andExpect(content().string(containsString("Request")));
        headers.set("Authorization", token2);
        mockMvc.perform(post("/requests/new").headers(headers).param("username", username3))
                .andDo(print()).andExpect(status().isOk()).andExpect(content().string(containsString("Request")));
        mockMvc.perform(put("/requests/accept").headers(headers).param("username", username1))
                .andDo(print()).andExpect(status().isOk()).andExpect(content().string(containsString("Request")));
        headers.set("Authorization", token3);
        mockMvc.perform(put("/requests/accept").headers(headers).param("username", username1))
                .andDo(print()).andExpect(status().isOk()).andExpect(content().string(containsString("Request")));
        mockMvc.perform(put("/requests/accept").headers(headers).param("username", username2))
                .andDo(print()).andExpect(status().isOk()).andExpect(content().string(containsString("Request")));
        String response = "online\":true},\"param\":500,\"blocked\":false},{\"user\":{";
        mockMvc.perform(get("/contact/all").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(response)));
        headers.set("Authorization", token2);
        mockMvc.perform(get("/contact/all").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(response)));
        headers.set("Authorization", token1);
        mockMvc.perform(get("/contact/all").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(response)));
    }

    @Test
    public void aGetMatchesFulfilledIsEmptyAtFirst() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token1);
        mockMvc.perform(get("/matches/fulfilled").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("{\"matches\":[]}")));
        headers.set("Authorization", token2);
        mockMvc.perform(get("/matches/fulfilled").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("{\"matches\":[]}")));
        headers.set("Authorization", token3);
        mockMvc.perform(get("/matches/fulfilled").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("{\"matches\":[]}")));
    }

    @Test
    public void aGetMatchesActiveIsEmptyAtFirst() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token1);
        mockMvc.perform(get("/matches/active").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("{\"active\":[],\"received\":[],\"sent\":[]}")));
        headers.set("Authorization", token2);
        mockMvc.perform(get("/matches/active").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("{\"active\":[],\"received\":[],\"sent\":[]}")));
        headers.set("Authorization", token3);
        mockMvc.perform(get("/matches/active").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("{\"active\":[],\"received\":[],\"sent\":[]}")));
    }

    @Test
    public void bGetMatchesActive() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token3);
        // Add location
        Map<String, Double> map = new HashMap<>();
        map.put("lat", 2.2);
        map.put("lon", 2.2);
        List<Map<String, Double>> mapList = new ArrayList<>();
        mapList.add(map);
        final StringWriter sw = new StringWriter();
        objectMapper.writeValue(sw, mapList);
        mockMvc.perform(post("/location/update").headers(headers).content(sw.toString())
                .contentType(MediaType.APPLICATION_JSON_VALUE));
        headers.set("Authorization", token2);
        mockMvc.perform(post("/location/update").headers(headers).content(sw.toString())
                .contentType(MediaType.APPLICATION_JSON_VALUE));
        headers.set("Authorization", token1);
        mockMvc.perform(post("/location/update").headers(headers).content(sw.toString())
                .contentType(MediaType.APPLICATION_JSON_VALUE));
        // Check matches
        String body = mockMvc.perform(get("/matches/active").headers(headers)).andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("{\"active\":[{\"id"))).andReturn().getResponse().getContentAsString();
        matchId12 = objectMapper.readValue(body, Matches.class).getActive().get(0).getId();
        matchId13 = objectMapper.readValue(body, Matches.class).getActive().get(1).getId();
        headers.set("Authorization", token2);
        body = mockMvc.perform(get("/matches/active").headers(headers)).andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("{\"active\":[{\"id"))).andReturn().getResponse().getContentAsString();
        matchId23 = objectMapper.readValue(body, Matches.class).getActive().get(0).getId();
        headers.set("Authorization", token3);
        mockMvc.perform(get("/matches/active").headers(headers)).andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("{\"active\":[{\"id"))).andReturn().getResponse().getContentAsString();
    }

    @Test
    public void cSendMatchRequestSent() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token1);
        mockMvc.perform(post("/match/send").headers(headers).param("matchId", matchId12 + ""))
                .andDo(print()).andExpect(status().isOk()).andExpect(content().string(containsString("Match request registered")));
        mockMvc.perform(post("/match/send").headers(headers).param("matchId", matchId13 + ""))
                .andDo(print()).andExpect(status().isOk()).andExpect(content().string(containsString("Match request registered")));
        mockMvc.perform(get("/matches/active").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("sent\":[{\"id")));
    }

    @Test
    public void cSendMatchRequestWrongMatchId() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token1);
        mockMvc.perform(post("/match/send").headers(headers).param("matchId", matchId23 + ""))
                .andDo(print()).andExpect(status().isInternalServerError()).andExpect(content().string(containsString("ERROR")));
    }

    @Test
    public void cSendMatchRequestIdNotFound() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token1);
        mockMvc.perform(post("/match/send").headers(headers).param("matchId", 99999 + ""))
                .andDo(print()).andExpect(status().isNotFound()).andExpect(content().string(containsString("No match with such id")));
    }

    @Test
    public void dSendMatchRequestAlreadyARequest() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token1);
        mockMvc.perform(post("/match/send").headers(headers).param("matchId", matchId12 + ""))
                .andDo(print()).andExpect(status().isConflict()).andExpect(content().string(containsString("already")));
        headers.add("Authorization", token2);
        mockMvc.perform(post("/match/send").headers(headers).param("matchId", matchId12 + ""))
                .andDo(print()).andExpect(status().isConflict()).andExpect(content().string(containsString("already")));
    }

    @Test
    public void dCancelNotSender() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token2);
        mockMvc.perform(delete("/match/cancel").headers(headers).param("matchId", matchId12 + ""))
                .andDo(print()).andExpect(status().isConflict()).andExpect(content().string(containsString("not the sender")));
        mockMvc.perform(get("/matches/active").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("received\":[{\"id")));
    }

    @Test
    public void eAcceptMatchRequestOnFulfilled() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token2);
        mockMvc.perform(put("/match/accept").headers(headers).param("matchId", matchId12 + ""))
                .andDo(print()).andExpect(status().isOk()).andExpect(content().string(containsString("")));
        mockMvc.perform(get("/matches/fulfilled").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString(",\"state\":\"accepted\"}")));
    }

    @Test
    public void fCancelAcceptedMatchRequest() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token1);
        mockMvc.perform(delete("/match/cancel").headers(headers).param("matchId", matchId12 + ""))
                .andDo(print()).andExpect(status().isOk()).andExpect(content().string(containsString("")));
        mockMvc.perform(get("/matches/active").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("{\"active\":[{\"id\":" + matchId12 + ",")));
    }

}
