package restapi;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.junit.Before;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.HttpHeaders;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.web.servlet.MockMvc;
import restapi.json.Token;
import restapi.services.FirebaseService;

import static org.hamcrest.Matchers.containsString;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;


@SpringBootTest
@AutoConfigureMockMvc
@TestMethodOrder(MethodOrderer.Alphanumeric.class)
public class UserSecuredEndpointTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private FirebaseService firebaseService;

    @Autowired
    private ObjectMapper objectMapper;

    private static String token;

    @Test
    public void AaSetUp() throws Exception {
        String username = "userSecured";
        String password = "pas";
        String body = mockMvc.perform(post("/public/users/signup")
                        .param("username", username)
                        .param("password", password)
                ).andDo(print()).andExpect(status().isCreated()).andReturn().getResponse().getContentAsString();
        token = objectMapper.readValue(body, Token.class).getToken();
        assert !token.isEmpty();
    }

    @Test
    public void aCurrent() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token);
        this.mockMvc.perform(get("/users/current").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("\"username\":\"userSecured\",")));
    }

    @Test
    public void aCurrentWrongToken() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", "token");
        this.mockMvc.perform(get("/users/current").headers(headers))
                .andDo(print()).andExpect(status().isUnauthorized());
    }

    @Test
    public void bLogout() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", token);
        this.mockMvc.perform(post("/users/logout").headers(headers))
                .andDo(print()).andExpect(status().isOk())
                .andExpect(content().string(containsString("Logged Out")));
        this.mockMvc.perform(get("/users/current").headers(headers))
                .andDo(print()).andExpect(status().isUnauthorized());
    }

    @Test
    public void bLogoutWrongToken() throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", "token");
        this.mockMvc.perform(get("/users/current").headers(headers))
                .andDo(print()).andExpect(status().isUnauthorized());
    }
}