import { Component, OnInit } from '@angular/core';
import { LoginService } from 'src/app/providers/auth/login.service';
import { StorageService } from 'src/app/providers/storage.service';

@Component({
  selector: 'app-login',
  templateUrl: './login.page.html',
  styleUrls: ['./login.page.scss'],
})
export class LoginPage implements OnInit {
  username: string;
  password: string;

  rememberMe: boolean;

  secretPassword = false;

  passwordType: string = 'password';
  passwordIcon: string = 'eye-off-outline';

  constructor(
    private loginService: LoginService,
    private storage: StorageService
  ) { }

  ngOnInit() {
    this.storage.get('autoPassword').then(pswd => {
      if (pswd != null) {
        this.storage.get('username').then(username => {
          if (username != null) {
            this.username = username;
            this.password = pswd;
            this.secretPassword = true;
            this.rememberMe = true;
          }
        });
      }
    });
  }

  onLogin() {
    if (this.username.indexOf('@') != -1) {
      this.loginService.emailLogin(this.username, this.password, this.rememberMe);
    } else if (this.username.match(/^[a-zA-Z]+$/)) {
      this.loginService.usernameLogin(this.username, this.password, this.rememberMe);
    } else {
      this.loginService.phoneLogin(this.username, this.password, this.rememberMe);
    }
  }

  hideShowPassword() {
    if (!this.secretPassword) {
      this.passwordType = this.passwordType === 'text' ? 'password' : 'text';
      this.passwordIcon = this.passwordIcon === 'eye-off-outline' ? 'eye-outline' : 'eye-off-outline';
    }
  }

  onChangePassword() {
    if (this.secretPassword) {
      this.password = '';
      this.secretPassword = false;
    }
  }
}
