import { Component, OnInit } from '@angular/core';
import { NgForm } from '@angular/forms';
import { Router } from '@angular/router';
//import { getAuth } from "firebase/auth";
import { UserData } from 'src/app/interfaces/data-classes';
import { ApiService } from 'src/app/providers/server/api.service';
import { FirebaseAuthService } from 'src/app/providers/server/firebase-auth.service';
import { LoginService } from 'src/app/providers/auth/login.service';
import { StorageService } from 'src/app/providers/storage.service';
import { ToastService } from 'src/app/providers/os/toast.service';

@Component({
  selector: 'app-sign-up',
  templateUrl: './sign-up.page.html',
  styleUrls: ['./sign-up.page.scss'],
})
export class SignUpPage implements OnInit {

  textMessage = 'Sign up on NEARME';

  username: string = '';
  usernameValid = true;
  email: string = '';
  emailValid = true;
  password: string = '';
  passwordValid: boolean;

  passwordType: string = 'password';
  passwordIcon: string = 'eye-off-outline';

  errorMessageUsername = '';
  errorMessageEmail = 'Email not valid';
  errorMessagePassword = '';

  constructor(
    public router: Router,
    public api: ApiService,
    public toastService: ToastService,
    public storage: StorageService,
    public firebaseAuthService: FirebaseAuthService,
    private loginService: LoginService
  ) { }

  ngOnInit() {
  }

  hideShowPassword() {
    this.passwordType = this.passwordType === 'text' ? 'password' : 'text';
    this.passwordIcon = this.passwordIcon === 'eye-off-outline' ? 'eye-outline' : 'eye-off-outline';
  }

  onSignup(form: NgForm) {
    if (form.valid) {
      this.storage.get('phoneNumber').then(phoneNumber =>
        this.storage.get('country').then(country => {
          var user = new UserData();
          user.username = this.username;
          user.password = this.password;
          user.email = this.email;
          if (phoneNumber != null) {
            user.phone = phoneNumber;
          }
          if (country != null) {
            user.country = country.code;
            user.authUID = this.firebaseAuthService.getCurrentUID();
          }
          this.api.signUp(user).subscribe({
            next: async response => {
              await this.storage.set('authToken', response.token);
              await this.storage.set('username', this.username);
              await this.loginService.signedin(this.password);
              this.toastService.presentToast('User registered');
              this.storage.remove('phoneNumber');
              this.storage.remove('countryCode');
              this.router.navigate(['/tabs'], { replaceUrl: true });
            },
            error: error => {
              console.log(error);
              switch (error.status) {
                case 409:
                  console.error(error.body);
                  console.error(error);
                  this.toastService.presentToast(error.body.message);
                  break;
              }
            }
          })
        })
      );
    }
  }

  onChangeUsername() {
    if (this.username.length < 5) {
      this.usernameValid = false;
      this.errorMessageUsername = 'Username must be at last 5 characters long';
    } else if (this.password.match(" ")) {
      this.passwordValid = false;
      this.errorMessageUsername = 'Username must not contain spaces';
    } else if (this.password.match("@")) {
      this.passwordValid = false;
      this.errorMessageUsername = 'Username must not contain the symbol @';
    } else {
      this.usernameValid = true;
    }
  }

  onChangeEmail() {
    if (!this.email.match('^[A-Z0-9a-z\\._%+-]+@([A-Za-z0-9-]+\\.)+[A-Za-z]{2,4}$')) {
      this.emailValid = false;
    } else {
      this.emailValid = true;
    }
  }

  passwordSecured() {
    if (this.password.length < 8) {
      this.passwordValid = false;
      this.errorMessagePassword = 'Password must be at least 8 symbols long';
    } else if (this.password.match(" ")) {
      this.passwordValid = false;
      this.errorMessagePassword = 'Password must not contain spaces';
    } else if (this.password.match("^(.{0,7}|[^0-9]*|[^A-Z]*|[^a-z]*|[a-zA-Z0-9]*)$")) {
      this.passwordValid = false;
      this.errorMessagePassword = 'Password must contain at least one uppercase letter, one lowercase letter, one number and one special character';
    } else {
      this.passwordValid = true;
    }
  }

}
