import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { UserData } from 'src/app/interfaces/data-classes';
import { ApiService } from 'src/app/providers/server/api.service';
import { StorageService } from 'src/app/providers/storage.service';
import { ToastService } from 'src/app/providers/os/toast.service';

@Component({
  selector: 'app-change-password',
  templateUrl: './change-password.page.html',
  styleUrls: ['./change-password.page.scss'],
})
export class ChangePasswordPage implements OnInit {

  oldPassword: string;
  newPassword1: string;
  newPassword2: string;

  passwordValid1 = true;
  errorMessagePassword1: string = 'Password must be at least 8 symbols long';
  passwordValid2 = true;
  errorMessagePassword2: string;

  constructor(
    private api: ApiService,
    private storage: StorageService,
    private toastService: ToastService,
    private router: Router
  ) { }

  ngOnInit() {
  }

  onAccept() {
    if (this.passwordValid1 && this.newPassword1 != '' && this.passwordValid2 && this.newPassword2 != '') {
      this.changePassword();
    }
  }

  password1Secured() {
    if (this.newPassword1.length < 8) {
      this.passwordValid1 = false;
      this.errorMessagePassword1 = 'Password must be at least 8 symbols long';
    } else if (this.newPassword1.match(" ")) {
      this.passwordValid1 = false;
      this.errorMessagePassword1 = 'Password must not contain spaces';
    } else if (this.newPassword1.match("^(.{0,7}|[^0-9]*|[^A-Z]*|[^a-z]*|[a-zA-Z0-9]*)$")) {
      this.passwordValid1 = false;
      this.errorMessagePassword1 = 'Password must contain at least one uppercase letter, one lowercase letter, one number and one special character';
    } else if (this.newPassword1 == this.oldPassword) {
      this.passwordValid1 = false;
      this.errorMessagePassword1 = 'Password must be different from the old';
    } else {
      this.passwordValid1 = true;
    }
  }

  password2Secured() {
    if (this.newPassword1 != this.newPassword2) {
      this.passwordValid2 = false;
      this.errorMessagePassword2 = 'Passwords must be the same';
    } else {
      this.passwordValid2 = true;
    }
  }

  changePassword() {
    this.storage.get('authToken').then(token => {
      this.api.changePassword(token, this.oldPassword, this.newPassword1).subscribe({
        next: async response => {
          this.router.navigate(['tabs/'], { replaceUrl: true });
        },
        error: error => {
          switch (error.status) {
            case 401:
              this.toastService.presentToast('Wrong password, try again');
              break;
            case 404:
              this.toastService.presentToast('No user with this username');
              break;
          }
        }
      });
    });
  }

}
