import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { NavController } from '@ionic/angular';
import { LocationData } from 'src/app/interfaces/data-classes';
import { ApiService } from 'src/app/providers/server/api.service';
import { LogoutService } from 'src/app/providers/auth/logout.service';
import { StorageService } from 'src/app/providers/storage.service';
import { WebSocketService } from 'src/app/providers/server/web-socket.service';
import { CommonUtilsService } from 'src/app/providers/common-utils.service';

@Component({
  selector: 'app-contact-list',
  templateUrl: './contact-list.page.html',
  styleUrls: ['./contact-list.page.scss'],
})
export class ContactListPage implements OnInit {

  contactList: any[];
  contactsToShowOnline: any[];
  contactsToShowOffline: any[];

  showSearchBar: boolean = true;
  queryText = '';

  constructor(
    public api: ApiService,
    public storage: StorageService,
    public logoutService: LogoutService,
    private router: Router,
    private utils: CommonUtilsService
    // TODO CHANGE
    //private webSocketAPI: WebSocketService
  ) { }

  ngOnInit() {

  }

  ionViewWillEnter() {
    this.loadContacts();
    // TODO CHANGE
    //this.connect()
  }

  doRefresh(event) {
    this.loadContacts();
    setTimeout(() => {
      event.target.complete();
    }, 500);
  }

  loadContacts() {
    this.storage.get('authToken').then(token => {
      this.api.getContacts(token).subscribe({
        next: response => {
          this.contactList = response.contacts;
          this.filterContacts();
        },
        error: error => {
          switch (error.status) {
            case 401:
              this.logoutService.logout();
              break;
          }
        }
      });
    })
  }

  showHideSearchBar() {
    this.showSearchBar = !this.showSearchBar;
  }

  filterContacts() {
    const queryWords = this.queryText.toLowerCase().replace(/,|\.|-/g, ' ').split(' ').filter(w => !!w.trim().length);
    this.contactsToShowOnline = [];
    this.contactsToShowOffline = [];
    if (queryWords.length) {
      this.contactList.forEach((contact: any) => {
        queryWords.forEach((queryWord: string) => {
          if ((contact.user.name != null && contact.user.name.toLowerCase().indexOf(queryWord) > -1) || contact.user.username.toLowerCase().indexOf(queryWord) > -1) {
            if (contact.user.online) {
              this.contactsToShowOnline.push(contact);
            } else {
              this.contactsToShowOffline.push(contact);
            }
          }
        });
      });
    } else {
      this.contactList.forEach((contact: any) => {
        if (contact.user.online) {
          this.contactsToShowOnline.push(contact);
        } else {
          this.contactsToShowOffline.push(contact);
        }
      });
    }
  }

  getAvatar(id) {
    return this.utils.getUserAvatarLink(id);
  }

  addContact() {
    this.router.navigate(['tabs/requests/find']);
  }

  onContactClick(contact) {
    this.storage.set('contact', contact).then(() =>
      this.router.navigate(['tabs/contacts/detail'])
    )
  }

}
