import { Component, OnInit } from '@angular/core';
import { ApiService } from 'src/app/providers/server/api.service';
import { LogoutService } from 'src/app/providers/auth/logout.service';
import { StorageService } from 'src/app/providers/storage.service';
import { Contacts, Contact } from '@capacitor-community/contacts'
import { ToastService } from 'src/app/providers/os/toast.service';
import { CommonUtilsService } from 'src/app/providers/common-utils.service';

@Component({
  selector: 'app-find-friends',
  templateUrl: './find-friends.page.html',
  styleUrls: ['./find-friends.page.scss'],
})
export class FindFriendsPage implements OnInit {

  private phones: string[] = [];
  private users: any[];

  private firstTime = true;

  constructor(
    private storage: StorageService,
    private api: ApiService,
    private logoutService: LogoutService,
    private toastService: ToastService,
    private utils: CommonUtilsService
  ) { }

  ngOnInit() {
    this.loadPhones();
  }

  loadPhones() {
    Contacts.getPermissions().then(permissionStatus => {
      if (permissionStatus.granted) {
        Contacts.getContacts().then(result => {
          result.contacts.forEach(contact => {
            contact.phoneNumbers.forEach(phone => this.phones.push(this.parsePhoneNumber(phone.number)));
          });
          this.areContactsFriends();
        });
      } else if (this.firstTime) {
        this.firstTime = false;
        this.loadPhones();
      } else {
        this.toastService.presentToast('Without permision we cannot access your contacts to find new friends.');
      }
    })
  }

  parsePhoneNumber(phone: string) {
    return phone.split(' ').join('');
  }

  areContactsFriends() {
    this.storage.get('authToken').then(token => {
      this.api.areContactsUsers(token, this.phones).subscribe({
        next: response => {
          this.users = response;
        },
        error: error => {
          switch (error.status) {
            case 401:
              this.logoutService.logout();
              break;
          }
        }
      });
    });
  }

  onSend(user: any) {
    this.storage.get('authToken').then(token => {
      this.api.requestNew(token, user.username).subscribe({
        next: response => {
          this.areContactsFriends();
        },
        error: error => {
          switch (error.status) {
            case 401:
              this.logoutService.logout();
              break;
          }
        }
      });
    });
  }

}