import { Component, OnInit } from '@angular/core';
import { ApiService } from 'src/app/providers/server/api.service';
import { LogoutService } from 'src/app/providers/auth/logout.service';
import { StorageService } from 'src/app/providers/storage.service';
import { CommonUtilsService } from 'src/app/providers/common-utils.service';

@Component({
  selector: 'app-friend-requests',
  templateUrl: './friend-requests.page.html',
  styleUrls: ['./friend-requests.page.scss'],
})
export class FriendRequestsPage implements OnInit {

  sent: any[];
  received: any[];

  constructor(
    public api: ApiService,
    public storage: StorageService,
    public logoutService: LogoutService,
    private utils: CommonUtilsService
  ) { }

  ngOnInit() {

  }

  ionViewWillEnter() {
    this.loadRequests();
  }

  doRefresh(event) {
    this.loadRequests();
    setTimeout(() => {
      event.target.complete();
    }, 500);
  }

  loadRequests() {
    this.storage.get('authToken').then(token => {
      this.api.getRequests(token).subscribe({
        next: response => {
          this.sent = response.sent;
          this.received = response.received;
        },
        error: error => {
          switch (error.status) {
            case 401:
              this.logoutService.logout();
              break;
          }
        }
      });
    });
  }

  onAccept(user: any) {
    this.storage.get('authToken').then(token => {
      this.api.requestAccept(token, user.user.username).subscribe({
        next: response => {
          this.loadRequests();
        },
        error: error => {
          switch (error.status) {
            case 401:
              this.logoutService.logout();
              break;
          }
        }
      });
    });
  }

  onReject(user: any) {
    this.storage.get('authToken').then(token => {
      this.api.requestReject(token, user.user.username).subscribe({
        next: response => {
          this.loadRequests();
        },
        error: error => {
          switch (error.status) {
            case 401:
              this.logoutService.logout();
              break;
          }
        }
      });
    });
  }

  onCancel(user: any) {
    this.storage.get('authToken').then(token => {
      this.api.requestCancel(token, user.user.username).subscribe({
        next: response => {
          this.loadRequests();
        },
        error: error => {
          switch (error.status) {
            case 401:
              this.logoutService.logout();
              break;
          }
        }
      });
    });
  }
}
