import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { LoginService } from 'src/app/providers/auth/login.service';
import { ApiService } from 'src/app/providers/server/api.service';
import { StorageService } from 'src/app/providers/storage.service';

@Component({
  selector: 'app-splash',
  templateUrl: './splash.page.html',
  styleUrls: ['./splash.page.scss'],
})
export class SplashPage implements OnInit {

  constructor(
    private api: ApiService,
    private storage: StorageService,
    private router: Router,
    private loginService: LoginService
  ) { }

  ngOnInit() {
    this.getLoggedIn();
  }

  getLoggedIn() {
    this.storage.get('authToken').then(token => {
      if (token != null) {
        this.api.getUserInfo(token).subscribe({
          next: response => {
            this.router.navigate(['tabs/'], { replaceUrl: true });
          },
          error: error => {
            this.login();
          }
        });
      } else {
        this.login();
      }
    });
  }

  login() {
    this.storage.get('autoPassword').then(pswd => {
      if (pswd != null) {
        this.storage.get('username').then(username => {
          if (username != null) {
            this.loginService.usernameAutoLogin(username, pswd);
          } else {
            this.router.navigate(['home'], { replaceUrl: true });
          }
        });
      } else {
        this.router.navigate(['home'], { replaceUrl: true });
      }
    });
  }
}
