import { Component, Input, OnInit } from '@angular/core';
import { AlertController, ModalController } from '@ionic/angular';
import { UserData } from 'src/app/interfaces/data-classes';
import { ApiService } from 'src/app/providers/server/api.service';
import { LogoutService } from 'src/app/providers/auth/logout.service';
import { StorageService } from 'src/app/providers/storage.service';

@Component({
  selector: 'app-user-detail-edit-modal',
  templateUrl: './user-detail-edit-modal.page.html',
  styleUrls: ['./user-detail-edit-modal.page.scss'],
})
export class UserDetailEditModal {

  // Data passed in by componentProps

  constructor(
    public modalController: ModalController,
    private storage: StorageService,
    private api: ApiService,
    private logoutService: LogoutService
  ) {

  }

  dismiss() {
    // using the injected ModalController this page
    // can "dismiss" itself and optionally pass back data
    this.modalController.dismiss({
      'dismissed': true
    });
  }

  onClick(image: number) {
    this.storage.get('authToken').then(token => {
      this.api.changeAvatar(token, image).subscribe({
        next: response => {
          this.dismiss();
        },
        error: error => {
          switch (error.status) {
            case 401:
              this.logoutService.logout();
              this.dismiss();
              break;
          }
        }
      });
    });
  }

}