import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { AlertController } from '@ionic/angular';
import { UserData } from '../../interfaces/data-classes';
import { ApiService } from '../server/api.service';
import { StorageService } from '../storage.service';
import { ToastService } from '../os/toast.service';

@Injectable({
  providedIn: 'root'
})
export class LoginService {

  constructor(
    private api: ApiService,
    private storage: StorageService,
    private router: Router,
    private alertCtrl: AlertController,
    private toastService: ToastService
  ) { }

  usernameAutoLogin(username: string, password: string) {
    var user = new UserData();
    user.username = username;
    user.password = password;
    this.api.loginUsername(user).subscribe({
      next: async response => {
        await this.storage.set('authToken', response.token);
        this.router.navigate(['tabs/'], { replaceUrl: true });
      }
    });
  }

  usernameLogin(username: string, password: string, savePassword: boolean) {
    var user = new UserData();
    user.username = username;
    user.password = password;
    this.api.loginUsername(user).subscribe({
      next: async response => {
        await this.storage.set('authToken', response.token);
        await this.storage.set('username', username);
        if (savePassword) {
          await this.storage.set('autoPassword', password);
        } else {
          this.storage.remove('autoPassword');
        }
        this.router.navigate(['tabs/'], { replaceUrl: true });
      },
      error: error => {
        switch (error.status) {
          case 401:
            this.toastService.presentToast('Wrong password, try again');
            break;
          case 404:
            this.toastService.presentToast('No user with this username');
            break;
        }
      }
    });
  }

  emailLogin(email: string, password: string, savePassword: boolean) {
    var user = new UserData();
    user.email = email;
    user.password = password;
    this.api.loginEmail(user).subscribe({
      next: response => {
        this.storage.set('authToken', response.token);
        this.storage.set('autoEmail', email);
        if (savePassword) {
          this.storage.set('autoPassword', password);
        } else {
          this.storage.remove('autoPassword');
        }
        this.router.navigate(['tabs/'], { replaceUrl: true });
      },
      error: error => {
        switch (error.status) {
          case 401:
            this.toastService.presentToast('Wrong password, try again');
            break;
          case 404:
            this.toastService.presentToast('No user with this email');
            break;
        }
      }
    });
  }

  phoneLogin(phone: string, password: string, savePassword: boolean) {
    var user = new UserData();
    user.phone = phone;
    user.password = password;
    this.api.loginPhone(user).subscribe({
      next: response => {
        this.storage.set('authToken', response.token);
        this.storage.set('autoPhone', phone);
        if (savePassword) {
          this.storage.set('autoPassword', password);
        } else {
          this.storage.remove('autoPassword');
        }
        this.router.navigate(['tabs/'], { replaceUrl: true });
      },
      error: error => {
        switch (error.status) {
          case 401:
            this.toastService.presentToast('Wrong password, try again');
            break;
          case 404:
            this.toastService.presentToast('No user with this phone number');
            break;
        }
      }
    });
  }

  async signedin(password: string) {

    const alert = await this.alertCtrl.create({
      header: 'Save password',
      message: 'Do you want to activate autologin?',
      buttons: [
        {
          text: 'No',
          cssClass: 'secondary',
          handler: () => {}
        },
        {
          text: 'Yes',
          cssClass: 'primary',
          handler: () => {
            this.storage.set('password', password);
          }
        }
      ]
    });
    await alert.present();
  }
}
