import { Injectable } from '@angular/core';
import { Geolocation } from '@capacitor/geolocation';
import { LocationData } from '../../interfaces/data-classes';
import { ApiService } from '../server/api.service';
import { StorageService } from '../storage.service';
import { ToastService } from './toast.service';
import { WebSocketService } from '../server/web-socket.service';
//import BackgroundGeolocation from '@mauron85/cordova-plugin-background-geolocation';
import { BackgroundGeolocation, BackgroundGeolocationConfig, BackgroundGeolocationEvents, BackgroundGeolocationResponse } from '@awesome-cordova-plugins/background-geolocation/ngx';
//import { registerPlugin, Plugins } from "@capacitor/core";
//const { App, BackgroundTask } = Plugins;
//import { BackgroundGeolocationPlugin } from "@capacitor-community/background-geolocation";
//const BackgroundGeolocation = registerPlugin<BackgroundGeolocationPlugin>("BackgroundGeolocation");

@Injectable({
  providedIn: 'root'
})
export class BackgroundService {

  username: string;
  token: string;

  constructor(
    private toastService: ToastService,
    private websocket: WebSocketService,
    private storage: StorageService,
    private api: ApiService,
    private backgroundGeolocation: BackgroundGeolocation
  ) { }

  //*
  async getCurrentPosition() {
    const coordinates = await Geolocation.getCurrentPosition();
    console.log('Current position:', coordinates);
    //this.toastService.presentToast('latitude: ' + coordinates.coords.latitude + ', longitude: ' + coordinates.coords.longitude);
    return coordinates.coords;
  }//*/

  async initBackground() {
    this.storage.get('username').then(username => this.username = username);
    this.storage.get('authToken').then(token => this.token = token);

    const config2: BackgroundGeolocationConfig = {
      locationProvider: BackgroundGeolocation.getPlugin().ACTIVITY_PROVIDER, //	Set location provider @see PROVIDERS
      desiredAccuracy: BackgroundGeolocation.getPlugin().HIGH_ACCURACY, // Desired accuracy in meters. Possible values [HIGH_ACCURACY, MEDIUM_ACCURACY, LOW_ACCURACY, PASSIVE_ACCURACY]. Accuracy has direct effect on power drain. Lower accuracy = lower power drain.
      stationaryRadius: 50, // Stationary radius in meters. When stopped, the minimum distance the device must move beyond the stationary location for aggressive background-tracking to engage.
      distanceFilter: 50, // The minimum distance (measured in meters) a device must move horizontally before an update event is generated. @see Apple docs.
      notificationTitle: 'Background tracking', // Custom notification title in the drawer. (goes with startForeground)
      notificationText: 'enabled', // Custom notification text in the drawer. (goes with startForeground)
      debug: true, // When enabled, the plugin will emit sounds for life-cycle events of background-geolocation! See debugging sounds table.
      interval: 10000, // Android // The minimum time interval between location updates in milliseconds. @see Android docs for more information.
      fastestInterval: 5000, // Android // Fastest rate in milliseconds at which your app can handle location updates. @see Android docs.
      activitiesInterval: 10000, // Android // Rate in milliseconds at which activity recognition occurs. Larger values will result in fewer activity detections while improving battery life.
      url: 'http://192.168.81.15:3000/location', // Server url where to send HTTP POST with recorded locations @see HTTP locations posting
      httpHeaders: { // Optional HTTP headers sent along in HTTP request
        'X-FOO': 'bar'
      },
      // customize post properties
      postTemplate: { // Customization post template @see Custom post template	
        lat: '@latitude',
        lon: '@longitude',
        foo: 'bar' // you can also add your own properties
      },
      // From now on defaults:
      stopOnTerminate: true, // Enable this in order to force a stop() when the application terminated (e.g. on iOS, double-tap home button, swipe away the app).
      startOnBoot: false, // Android // Start background service on device boot.	
      stopOnStillActivity: true, // Android // @deprecated stop location updates, when the STILL activity is detected	
      notificationsEnabled: true, // Android // Enable/disable local notifications when tracking and syncing locations
      startForeground: false, // Android // Allow location sync service to run in foreground state. Foreground state also requires a notification to be presented to the user.	
      notificationIconColor: '#4CAF50', // Android // The accent color to use for notification. Eg. #4CAF50. (goes with startForeground)
      notificationIconLarge: '', // Android // The filename of a custom notification icon. @see Android quirks. (goes with startForeground)
      notificationIconSmall: '',  // Android // The filename of a custom notification icon. @see Android quirks. (goes with startForeground)
      activityType: 'OtherNavigation', // iOS // [AutomotiveNavigation, OtherNavigation, Fitness, Other] Presumably, this affects iOS GPS algorithm. @see Apple docs for more information
      pauseLocationUpdates: false, // iOS // Pauses location updates when app is paused. *@see Apple docs
      saveBatteryOnBackground: false, // iOS // Switch to less accurate significant changes and region monitory when in background
      syncUrl: '', // Server url where to send fail to post locations @see HTTP locations posting	
      syncThreshold: 100, // Specifies how many previously failed locations will be sent to server at once	
      maxLocations: 10000, // Limit maximum number of locations stored into db	
    };
    const config: BackgroundGeolocationConfig = {
      desiredAccuracy: 10,
      stationaryRadius: 50,
      distanceFilter: 30,
      debug: false, //  enable this hear sounds for background-geolocation life-cycle.
      stopOnTerminate: false, // enable this to clear background location settings when the app terminates
      //notificationsEnabled: false // Enable/disable local notifications when tracking and syncing locations
      startForeground: false, // Allow location sync service to run in foreground state. Foreground state also requires a notification to be presented to the user.
      notificationTitle: 'Background Geolocation', // Custom notification title in the drawer. (goes with startForeground)
      notificationText: 'activated', 
      locationProvider: BackgroundGeolocation.getPlugin().DISTANCE_FILTER_PROVIDER, //	Set location provider @see PROVIDERS
      interval: 10 * 60 * 1000, // Android // The minimum time interval between location updates in milliseconds. @see Android docs for more information.
      fastestInterval: 60 * 1000, // Android // Fastest rate in milliseconds at which your app can handle location updates. @see Android docs.
      activitiesInterval: 60 * 1000, // Android // Rate in milliseconds at which activity recognition occurs. Larger values will result in fewer activity detections while improving battery life.
      saveBatteryOnBackground: false, // iOS // Switch to less accurate significant changes and region monitory when in background
      url: 'https://nearby-apirest-deploy.herokuapp.com/location/update',
      httpHeaders: { // Optional HTTP headers sent along in HTTP request
        'Authorization': await this.storage.get('authToken')
      },
      postTemplate:
      {
        lat: '@latitude',
        lon: '@longitude',
      },//*/
    };

    this.backgroundGeolocation.configure(config).then(() => {

      //console.log('inside then after config, i am guessing after config set');

      this.backgroundGeolocation.on(BackgroundGeolocationEvents.location).subscribe((location: BackgroundGeolocationResponse) => {

        //console.log('inside subscribe after on, i am guessing after get a new location');

        console.log(location);

        /*if (!this.websocket.checkConnection()) {
          this.websocket.connect();
        }//*/

        /*
        this.toastService.presentToast('on background iteration: before greeting');

        this.api.loadGreeting();

        this.toastService.presentToast('on background iteration: after greeting');

        this.storage.get('username').then(username => {
          this.websocket.updateLocation(new LocationData(location.latitude.toString(), location.longitude.toString(), username));
        });

        this.toastService.presentToast('on background iteration: before finish');
        //*/

        // IMPORTANT:  You must execute the finish method here to inform the native plugin that you're finished,
        // and the background-task may be completed.  You must do this regardless if your operations are successful or not.
        // IF YOU DON'T, ios will CRASH YOUR APP for spending too much time in the background.
        this.backgroundGeolocation.finish(); // FOR IOS ONLY

        //this.toastService.presentToast('on background iteration: after finish');
      });


      //console.log('inside then after config, i am guessing after on location');

    });

    // start recording location
    this.backgroundGeolocation.start();

  }

  stopBackgroundFetch() {
    // If you wish to turn OFF background-tracking, call the #stop method.
    this.backgroundGeolocation.stop();

  }

  /*
    initBackground() {
      App.addListener('appStateChange', state => {
        if (!state.isActive) {
          // The app has become inactive. We should check if we have some work left to do, and, if so,
          // execute a background task that will allow us to finish that work before the OS
          // suspends or terminates our app:
  
          let taskId = BackgroundTask.beforeExit(async () => {
            // In this function We might finish an upload, let a network request
            // finish, persist some data, or perform some other task
  
            this.getCurrentPosition();
  
            // Must call in order to end our task otherwise
            // we risk our app being terminated, and possibly
            // being labeled as impacting battery life
            BackgroundTask.finish({
              taskId,
            });
          });
        }
      });
    }//*/
  /*
  initBackgroundFetch() {
    BackgroundGeolocation.addWatcher(
      {
        backgroundMessage: "Cancel to prevent battery drain",
        backgroundTitle: "Using your location",
        requestPermissions: true,
        stale: false,
        distanceFilter: 50
      },
      function callback(location, error) {
        if (error) {
          if (error.code === "NOT_AUTHORIZED") {
            if (window.confirm(
              "This app needs your location, " +
              "but does not have permission.\n\n" +
              "Open settings now?"
            )) {
              BackgroundGeolocation.openSettings();
            }
          }
          return console.error(error);
        }
        this.sendLocation(location);
        return console.log(location);
      }
    ).then(function afterTheWatcherHasBeenAdded(watcherId) {
      this.watcherId = watcherId;
    });
  }

  async sendLocation(location) {
    const username = await this.storage.get('username');
    this.api.loadGreeting();
    this.websocket.updateLocation(new LocationData(location.latitude.toString(), location.longitude.toString(), username));
  }

  stopTraking() {
    BackgroundGeolocation.removeWatcher({
      id: this.watcherId
    });
  }
  //*/

  /*
  async initBackgroundFetch() {
    const status = await BackgroundFetch.configure({
      minimumFetchInterval: 15
    }, async (taskId) => {
      console.log('[BackgroundFetch] EVENT:', taskId);
      // Perform your work in an awaited Promise
      const result = await this.performYourWorkHere();
      console.log('[BackgroundFetch] work complete:', result);
      // [REQUIRED] Signal to the OS that your work is complete.
      BackgroundFetch.finish(taskId);
    }, async (taskId) => {
      // The OS has signalled that your remaining background-time has expired.
      // You must immediately complete your work and signal #finish.
      console.log('[BackgroundFetch] TIMEOUT:', taskId);
      // [REQUIRED] Signal to the OS that your work is complete.
      BackgroundFetch.finish(taskId);
    });

    // Checking BackgroundFetch status:
    if (status !== BackgroundFetch.STATUS_AVAILABLE) {
      // Uh-oh:  we have a problem:
      if (status === BackgroundFetch.STATUS_DENIED) {
        alert('The user explicitly disabled background behavior for this app or for the whole system.');
      } else if (status === BackgroundFetch.STATUS_RESTRICTED) {
        alert('Background updates are unavailable and the user cannot enable them again.')
      }
    }
  }
  //*
  async performYourWorkHere() {
    return new Promise((resolve, reject) => {
      setTimeout(async () => {
        const username = await this.storage.get('username');
        this.api.loadGreeting();
        this.getCurrentPosition().then((coords) => {
          this.websocket.updateLocation(new LocationData(coords.latitude.toString(), coords.longitude.toString(), username));
        }).catch(error => reject(error));
        resolve(true);
      }, 5000);
    });
  }
  //*/

}
