import { HttpClient, HttpHeaders, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { map } from 'rxjs/operators';
import { DataClass, LocationData } from '../../interfaces/data-classes';

@Injectable({
  providedIn: 'root'
})
export class ApiService {
  public apiURL = "https://nearby-apirest-deploy.herokuapp.com/";
  //public apiURL = "http://localhost:8080/";

  constructor(
    private http: HttpClient
  ) { }

  loadGreeting(): any {
    return this.http.get(this.apiURL + 'public/greeting')
      .pipe(map(this.processData, this));
  }

  processData(data: any) {
    return data.content;
  }

  getCsrf(token: string): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    return this.http.request('get', this.apiURL + 'csrf', { headers: headers });
  }

  /*******  PUBLIC / USERS  *******/

  signUp(dataClass: DataClass): any {
    const params = new HttpParams({ fromObject: dataClass.toDictionary() });
    return this.http.request('post', this.apiURL + 'public/users/signup', { params: params });
  }

  login(endpoint: string, dataClass: DataClass): any {
    const params = new HttpParams({ fromObject: dataClass.toDictionary() });
    return this.http.request('post', this.apiURL + 'public/users/login' + endpoint, { params: params });
  }

  loginToken(dataClass: DataClass): any {
    return this.login('token', dataClass);
  }

  loginUsername(dataClass: DataClass): any {
    return this.login('', dataClass);
  }

  loginEmail(dataClass: DataClass): any {
    return this.login('email', dataClass);
  }

  loginPhone(dataClass: DataClass): any {
    return this.login('phone', dataClass);
  }

  /***********  USERS  ************/

  logout(token: string): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    return this.http.request('post', this.apiURL + 'users/logout', { headers: headers });
  }

  getUserInfo(token: string): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    return this.http.request('get', this.apiURL + 'users/current', { headers: headers });
  }

  editUserInfo(token: string, dataClass: DataClass): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    const params = new HttpParams({ fromObject: dataClass.toDictionary() });
    return this.http.request('put', this.apiURL + 'users/editInfo', { headers: headers, params: params });
  }

  changePassword(token: string, old: string, password: string): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    const params = new HttpParams({ fromObject: { 'old': old, 'password': password } });
    return this.http.request('put', this.apiURL + 'users/psw', { headers: headers, params: params });
  }

  changeAvatar(token: string, avatar: number): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    const params = new HttpParams({ fromObject: { 'avatar': avatar } });
    return this.http.request('put', this.apiURL + 'users/avatar', { headers: headers, params: params });
  }

  /**********  CONTACTS  **********/

  getContacts(token: string): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    return this.http.request('get', this.apiURL + 'contact/all', { headers: headers });
  }

  getContactDetails(token:string, contactId:number): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    const params = new HttpParams({ fromObject: { 'contactId': contactId } });
    return this.http.request('get', this.apiURL + 'contact/details', { headers: headers, params: params });
  }

  areContactsUsers(token: string, phones: string[]): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    const params = new HttpParams({ fromObject: { 'phones': phones } });
    return this.http.request('get', this.apiURL + 'contacts/exist', { headers: headers, params: params });
  }

  updateRange(token: string, data: DataClass): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    const params = new HttpParams({ fromObject: data.toDictionary() });
    return this.http.request('put', this.apiURL + 'contact/range', { headers: headers, params: params });
  }

  updateBlocked(token: string, data: DataClass): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    const params = new HttpParams({ fromObject: data.toDictionary() });
    return this.http.request('put', this.apiURL + 'contact/block', { headers: headers, params: params });
  }

  /**********  REQUESTS  **********/

  getRequests(token: string): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    return this.http.request('get', this.apiURL + 'requests/mine', { headers: headers });
  }

  getOptions(token: string, username: string) {
    const headers = new HttpHeaders({ 'Authorization': token });
    const params = new HttpParams({ fromObject: { 'username': username } });
    return { headers: headers, params: params };
  }

  requestNew(token: string, username: string): any {
    return this.http.request('post', this.apiURL + 'requests/new', this.getOptions(token, username));
  }

  requestAccept(token: string, username: string): any {
    return this.http.request('put', this.apiURL + 'requests/accept', this.getOptions(token, username));
  }

  requestReject(token: string, username: string): any {
    return this.http.request('put', this.apiURL + 'requests/reject', this.getOptions(token, username));
  }

  requestCancel(token: string, username: string): any {
    return this.http.request('delete', this.apiURL + 'requests/cancel', this.getOptions(token, username));
  }

  requestSeen(token: string, username: string): any {
    return this.http.request('put', this.apiURL + 'requests/seen', this.getOptions(token, username));
  }

  /**********  LOCATION  **********/

  updateLocation(token: string, dataClass: DataClass): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    const params = new HttpParams({ fromObject: dataClass.toDictionary() });
    return this.http.request('put', this.apiURL + 'location/update', { headers: headers, params: params });
  }

  getLastMatches(token: string): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    return this.http.request('get', this.apiURL + 'matches/last', { headers: headers });
  }

  getActiveMatches(token: string): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    return this.http.request('get', this.apiURL + 'matches/active', { headers: headers });
  }

  getFulfilledMatches(token: string): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    return this.http.request('get', this.apiURL + 'matches/fulfilled', { headers: headers });
  }
  /********  MATCH REQUEST  *******/

  newMatchRequest(token: string, matchId: number): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    const params = new HttpParams({ fromObject: { 'matchId': matchId } });
    return this.http.request('post', this.apiURL + 'match/send', { headers: headers, params: params });
  }

  acceptMatchRequest(token: string, matchId: number): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    const params = new HttpParams({ fromObject: { 'matchId': matchId } });
    return this.http.request('put', this.apiURL + 'match/accept', { headers: headers, params: params });
  }

  rejectMatchRequest(token: string, matchId: number): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    const params = new HttpParams({ fromObject: { 'matchId': matchId } });
    return this.http.request('post', this.apiURL + 'match/reject', { headers: headers, params: params });
  }

  omitMatchRequest(token: string, matchId: number): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    const params = new HttpParams({ fromObject: { 'matchId': matchId } });
    return this.http.request('post', this.apiURL + 'match/omit', { headers: headers, params: params });
  }

  cancelMatchRequest(token: string, matchId: number): any {
    const headers = new HttpHeaders({ 'Authorization': token });
    const params = new HttpParams({ fromObject: { 'matchId': matchId } });
    return this.http.request('delete', this.apiURL + 'match/cancel', { headers: headers, params: params });
  }

}
