import { Injectable, NgZone } from '@angular/core';
import { Router } from '@angular/router';
import { UserData } from '../../interfaces/data-classes';
import { ApiService } from './api.service';
import { ToastService } from '../os/toast.service';
import { StorageService } from '../storage.service';

import { FirebaseAuthentication } from '@ionic-native/firebase-authentication/ngx';

@Injectable({
  providedIn: 'root'
})
export class FirebaseAuthService {
  verificationID: any;
  uid: string;

  constructor(
    private router: Router,
    private api: ApiService,
    private toast: ToastService,
    private storage: StorageService,
    private firebaseAuthentication: FirebaseAuthentication,
    private ngZone: NgZone
  ) { }

  public getCurrentUID() {
    return this.uid;
  }

  public signInWithPhoneNumber(phoneNumber) {
    //*
    this.firebaseAuthentication.verifyPhoneNumber(phoneNumber, 300)
      .then(verificationID => {
        this.verificationID = verificationID;
        this.router.navigate(['/verify'], { replaceUrl: true });
      }).catch(err => {
        console.error("phoner number verification failed", err);
      })//*/
  }

  public enterVerificationCode(code) {

    this.firebaseAuthentication.onAuthStateChanged().subscribe({
      next: userInfo => {
        console.log(userInfo);
        if (userInfo) {
          var user = new UserData();
          this.uid = userInfo.uid;
          user.authUID = userInfo.uid;
          this.ngZone.run(() => this.api.loginToken(user).subscribe({
            next: async response => {
              await this.storage.set('authToken', response.token);
              await this.storage.set('username', '');
              this.router.navigate(['tabs/'], { replaceUrl: true });
            },
            error: error => {
              switch (error.status) {
                case 404:
                  this.router.navigate(['/signup'], { replaceUrl: true });
                  break;
              }
            }
          }));
        } else {
          // user was signed out
        }
      },
      error: error => {
        console.error(error);
      }
    });//*/

    this.firebaseAuthentication.signInWithVerificationId(this.verificationID, code);
    //*/
  }

}
