import { Injectable } from '@angular/core';
//import * as Stomp from 'stompjs';
import { Client, Message } from '@stomp/stompjs';
import * as SockJS from 'sockjs-client';
import { LocationData, MatchData, MessageData, UserData } from '../../interfaces/data-classes';
import { ToastService } from '../os/toast.service';
import { ApiService } from './api.service';
import { StorageService } from '../storage.service';
import { LocalNotificationService } from '../os/local-notification.service';
import { LocationPage } from '../../pages/location/location.page';

@Injectable({
  providedIn: 'root'
})
export class WebSocketService {
  private apiURL = "";
  private webSocketEndPoint = 'public/socket/';
  preQueue: string = '/user/';
  postQueue: string = 'queue/matches';
  queue: string;
  client: any;

  userId: string;
  username: string;
  token: string;
  csrfHeaderName: string;
  csrfToken: string;

  locationPage: LocationPage;

  lastReadTime: Date;

  constructor(
    private storage: StorageService,
    private toastService: ToastService,
    private localNotifications: LocalNotificationService,
    api: ApiService
  ) {
    this.apiURL = api.apiURL;
  }

  initConnection(id, token) {
    if (id != null) { this.username = id; }
    if (token != null) { this.token = token; }
    this.connect();
  }

  connect() {
    console.log('Initialize WebSocket Connection');
    this.client = new Client({
      //brokerURL: this.apiURL + this.webSocketEndPoint,
      connectHeaders: {
        login: this.username,
        passcode: this.token,
      },
      debug: function (str) {
        console.log(str);
      },
      reconnectDelay: 20000,
      heartbeatIncoming: 4000,
      heartbeatOutgoing: 4000,
      webSocketFactory: () => {
        console.log("Sending Web-Socket Connection Petition To Server Through SockJs. username: " + this.username + ", token: " + this.token);
        return new SockJS(this.apiURL + this.webSocketEndPoint);
      }
    });
    
    this.client.onConnect = frame => {
      // Do something, all subscribes must be done is this callback
      // This is needed because this will be executed after a (re)connect
      this.client.subscribe('/user/queue/matches', (match) => {
        this.onMatchReceived(match);
      });
    };
    
    this.client.onStompError = frame => {
      // Will be invoked in case of error encountered at Broker
      // Bad login/passcode typically will cause an error
      // Complaint brokers will set `message` header with a brief message. Body may contain details.
      // Compliant brokers will terminate the connection after any error
      console.log('Broker reported error: ' + frame.headers['message']);
      console.log('Additional details: ' + frame.body);
    };
    
    this.client.activate();
  };

  disconnect() {
    if (this.client != null) {
      this.client.deactivate();
    }
    console.log('Disconnected');
  }

  /********** LOCATION **********/

  onMatchReceived(message) {
    /*
    const locMatch = JSON.parse(message.body);
    console.log('Message Recieved from Server :: ' + locMatch);
    //const s = locMatch.user.name + ' is ' + locMatch.distanceRange + 'm from you';
    //this.toastService.presentToast(s);
    if (this.locationPage != null) {
      this.locationPage.addMatch(this.getMatchFromResponse(locMatch));
    }
    //*/
    const now = new Date();
    if (this.lastReadTime == null || now.getTime() > this.lastReadTime.getTime() + 10000) {
      this.lastReadTime = now;
      if (this.locationPage != null) {
        this.locationPage.loadMatches();
      }
      const locMatch = JSON.parse(message.body);
      switch(locMatch.state) {
        case 'active':
          this.localNotifications.wantsToConnect(locMatch.user.username, locMatch.user.name);
          break;
        case 'unrequested':
          this.localNotifications.newMatch(locMatch.user.username, locMatch.user.name);
          break;
        default:
          console.log("Match recieved through web socket with unknown state. State = " + locMatch.state);
          break;
      } 
    }
  }

}
